#include<display.h>

SSD1306Wire display(0x3c, SDA, SCL);

void display_setup() {
    display.init();

    // clear the display
    display.clear();
//demo
    display.flipScreenVertically();
    display.setFont(ArialMT_Plain_10);

    display.setTextAlignment(TEXT_ALIGN_LEFT);
    // display.setFont(ArialMT_Plain_10);
    // display.drawString(0, 0, "Hello world");
    // display.setFont(ArialMT_Plain_16);
    // display.drawString(0, 10, "Hello world");
    display.setFont(ArialMT_Plain_24);
    display.drawString(0, 0, "smartPOC");
        
//end demo
    // display.setFont(ArialMT_Plain_10);
    // display.setTextAlignment(TEXT_ALIGN_RIGHT);
    // display.drawString(128, 54, String(millis()));
    // write the buffer to the display
    display.display();
}

bool displayEnableNotifications;
unsigned long lastNotificationTime = 0;
const char *displayStateText;
const char *displayErrorText;
unsigned long activeDisplayTimeLeft = 0;

// POCSAG TX LIVE DATA transmitterOffset
void display_update_state(const char *text) {
    displayEnableNotifications = true;
    displayStateText = text;
    lastNotificationTime = millis();
    display_update_state_draw();
}
void display_update_state_draw() {
    display.clear();
    display.setFont(ArialMT_Plain_24);
    display.drawString(0, 0, "smartPOC");
    display.setFont(ArialMT_Plain_16);
    display.drawString(0, 24, "starting up:");
    display.setFont(ArialMT_Plain_10);
    display.drawString(0, 40, displayStateText);
    display.display();
}

char bufferFreqText[10];
void display_idle(cfg_t *cfg_ref, char *timeStr) {
    display.clear();
    display.setFont(ArialMT_Plain_16);
    display.setTextAlignment(TEXT_ALIGN_CENTER);
    display.drawString(64, 0, "smartPOC");
    display.drawLine(0, 17, 128, 17);
    display.setTextAlignment(TEXT_ALIGN_LEFT);
    //
    sprintf(bufferFreqText, " %.3f", cfg_ref->tx_freq, cfg_ref->tx_baud);
    display.drawString(0, 16, bufferFreqText);
    //
    sprintf(bufferFreqText, "%dbd", cfg_ref->tx_baud);
    display.setTextAlignment(TEXT_ALIGN_RIGHT);
    display.drawString(128, 16, bufferFreqText);
    //
    sprintf(bufferFreqText, "%s", cfg_ref->device_name);
    display.setTextAlignment(TEXT_ALIGN_LEFT);
    display.setFont(ArialMT_Plain_10);
    display.drawString(0, 52, bufferFreqText);

    if (WiFi.isConnected()) {
        display.setFont(ArialMT_Plain_10);
        display.drawString(0, 32, WiFi.localIP().toString());
        display.setFont(ArialMT_Plain_10);
        display.drawString(0, 42, timeStr);
    } else {
    }
    display.display();
    //display.sendCommand(0xA3);
}
char bufferTransmitterText[40];
void display_transmit(POCSAGTransmitter *transmitter) {
    display.clear();
    display.setFont(ArialMT_Plain_24);
    display.drawString(0, 0, "smartPOC");
    display.setFont(ArialMT_Plain_16);
    display.drawString(0, 24, "transmitting");

    uint32_t curWord = transmitter->getTransmitWord();
    switch (curWord) {
        case 0x7A89C197: // idle
            sprintf(bufferTransmitterText, "<I D L E> %3d / %3d", transmitter->getTransmitPos(), transmitter->getTransmitLength());
        break;
        case 0x7CD215D8: // sync
            sprintf(bufferTransmitterText, "< SYNC > %3d / %3d", transmitter->getTransmitPos(), transmitter->getTransmitLength());
        break;
        default:
            sprintf(bufferTransmitterText, "%8X %3d / %3d", transmitter->getTransmitWord(), transmitter->getTransmitPos(), transmitter->getTransmitLength());
        break;
    }
    
    display.setFont(ArialMT_Plain_10);
    display.drawString(0, 48, bufferTransmitterText);
    display.display();
}
void display_report(const char *text) {

}
void display_loop(bool transmitterActive, POCSAGTransmitter *transmitter, cfg_t *cfg_ref, char *timeStr) {
    if (transmitterActive) {
        activeDisplayTimeLeft = millis();
    }

    if (cfg_ref->oled_timeout > 0) {
        if (activeDisplayTimeLeft + (cfg_ref->oled_timeout * 60e3) < millis()) {
            display.displayOff();
        } else {
            display.displayOn();
        }
    }
    if (transmitterActive)
        display_transmit(transmitter);
    else
        display_idle(cfg_ref, timeStr);
}

// local time, ip addr, transmitter name?