#include "mowas.h"

String mowasDWDMessageIDPrefix = "dwd.";
MoWaSClient::MoWaSClient(WiFiClient *client)
{
    _client = client;
}

void MoWaSClient::begin(int intervalMinutes, String region)
{
    this->dashboardUrl = "https://warnung.bund.de/api31/dashboard/";
    this->dashboardUrl.concat(region);
    this->dashboardUrl.concat(".json");

    this->checkInterval = 60e3 * intervalMinutes;
    Serial.printf("[MoWas]: check Interval : %dms\n", this->checkInterval);
    this->check();
}
void MoWaSClient::loop()
{
    if (millis() - this->lastCheck >= this->checkInterval)
    {
        // retry once
        if (!this->check())
            this->check();
        this->lastCheck = millis();
    }
}
bool MoWaSClient::isDirty()
{
    if (this->_dirty)
    {
        this->_dirty = false;
        return true;
    }
    return false;
}
bool MoWaSClient::check()
{
    Serial.println("[MoWaS]: checking " + this->dashboardUrl);
    this->_http.begin(this->dashboardUrl);

    int httpCode = this->_http.GET();

    if (httpCode > 0)
    {
        this->dashboardData = this->_http.getString();
        if (this->dashboardData.length() == 0)
        {
            Serial.println("Error: Empty JSON response!");
            return true;
        }
        this->currentMessage = this->parseDashboard(this->dashboardData);
        Serial.println("[MoWaS]: Current Message = " + this->currentMessage);
        if (this->currentMessage.equals(this->lastMessage) == false)
        {
            this->_dirty = true;
            this->lastMessage = this->currentMessage;
        }
        this->_http.end();
        return true;
    }
    else
    {
        Serial.printf("HTTP request failed, error: %s\n", this->_http.errorToString(httpCode).c_str());
        this->_http.end();
        return false;
    }
}
String MoWaSClient::parseDashboard(String jsonData)
{
    JsonDocument doc;
    deserializeJson(doc, jsonData);
    if (doc.size() > 0) {
        String alarmID = doc[0]["id"];
        // Ignore MoWaS alarms with DWD
        if (alarmID.indexOf(mowasDWDMessageIDPrefix) == 0) return emptyString;

        return doc[0]["payload"]["data"]["headline"];
    }
    return emptyString;
}
